local coveredBridge = ...

--------------------------------------------------------------------------

-- Register IDs for all prefab nodes referenced here
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/CenterPart", CoveredBridgePrefix .. "_CENTER_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/RootPart", CoveredBridgePrefix .. "_CENTER_ROOT_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/StartSlopePart", CoveredBridgePrefix .. "_SLOPE_START_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/EndSlopePart", CoveredBridgePrefix .. "_SLOPE_END_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/StartPart", CoveredBridgePrefix .. "_START_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/EndPart", CoveredBridgePrefix .. "_END_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/TilingPart", CoveredBridgePrefix .. "_TILING_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/CenterCoveragePart", CoveredBridgePrefix .. "_CENTER_COVERAGE_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/StartEndCoveragePart", CoveredBridgePrefix .. "_START_END_COVERAGE_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeGate.fbx/Prefab/coveredBridge/Gate/step_0_RootPart", CoveredBridgePrefix .. "_GATE_ROOT_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeGate.fbx/Prefab/coveredBridge/Gate/step_3_TopPart", CoveredBridgePrefix .. "_GATE_TOP_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeGate.fbx/Prefab/coveredBridge/Gate/step_2_Tiling2Part", CoveredBridgePrefix .. "_GATE_TILING2_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeGate.fbx/Prefab/coveredBridge/Gate/step_1_Tiling3Part", CoveredBridgePrefix .. "_GATE_TILING3_PREFAB")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	coveredBridge:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
                Name = _nodePrefix .. "_NAME",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
	CoveredBridgePrefix .. "_GATE_TILING2",
	CoveredBridgePrefix .. "_GATE_TILING3"}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

--------------------------------------------------------------------------

-- Register bridge tiling (the pillar part that extends to the ground)
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_TILING_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_TILING_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		}
	},
})

-- Register bridge start part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_START_PART",
        Name = CoveredBridgePrefix .. "_START_NAME",
        BuildingZone = _BridgeBuildingZone,
	Mover = { DataType = _MoverStart },
	ConstructorData = _ConstructorDataStart,
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 5 }
		},
	},
})

-- Register bridge end part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_END_PART",
        Name = CoveredBridgePrefix .. "_END_NAME",
        BuildingZone = _BridgeBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER_BRIDGE" },
	ConstructorData = _ConstructorDataEnd,
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 5 }
		},
	},
})

-- Register bridge start slope part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_SLOPE_START_PART",
        Name = CoveredBridgePrefix .. "_SLOPE_START_NAME",
        BuildingZone = _BridgeBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_SLOPE_START_PREFAB",
		FillerList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		},
	},
})

-- Register bridge end slope part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_SLOPE_END_PART",
        Name = CoveredBridgePrefix .. "_SLOPE_END_NAME",
        BuildingZone = _BridgeBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_SLOPE_END_PREFAB",
		FillerList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		},
	},
})

-- Register bridge center top part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_CENTER_TOP_PART",
        Name = CoveredBridgePrefix .. "_CENTER_TOP_NAME",
        BuildingZone = _BridgeBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_CENTER_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 5 },
			{ Resource = "PLANK", Quantity = 1 }
		},
	},
})

-- Register the center root part (the scalable part that places multiple center top parts)
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_CENTER_CORE_PART",
        Name = CoveredBridgePrefix .. "_CENTER_CORE_NAME",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = CoveredBridgePrefix .. "_CENTER_ROOT_PREFAB",
		EndPart = CoveredBridgePrefix .. "_CENTER_TOP_PART",
		FillerList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 2 }
		},
	},
})

-- Register the core part (the part with the bridge mover)
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_CORE_PART",
	Name = CoveredBridgePrefix .. "_CORE_NAME",
	Mover = { DataType = _MoverCore },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BRIDGE",
		StartPart = CoveredBridgePrefix .. "_START_PART",
		EndPart = CoveredBridgePrefix .. "_END_PART",
		FillerList = {
			CoveredBridgePrefix .. "_CENTER_CORE_PART"
		},
		SlopeFillerList = {
			CoveredBridgePrefix .. "_SLOPE_END_PART"
		},
		SlopeStartFillerList = {
			CoveredBridgePrefix .. "_SLOPE_START_PART"
		},
		MinimumScale = 1,
	},
	AssetBuildingFunction = CoveredBridgePrefix .. "_FUNCTION"
})
coveredBridge:register({
		DataType = "BUILDING_FUNCTION_BRIDGE",
                Id = CoveredBridgePrefix .. "_FUNCTION"
})

-- Register the stone watch tower (gate) part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_GATE_ROOT_PART",
	Name = CoveredBridgePrefix .. "_GATE_ROOT_NAME",
	Description = CoveredBridgePrefix .. "_GATE_ROOT_DESC",
	Category = "CORE",
        BuildingZone = _GateBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = CoveredBridgePrefix .. "_GATE_ROOT_PREFAB",
		EndPart = CoveredBridgePrefix .. "_GATE_TOP_PART",
		FillerList = {
			CoveredBridgePrefix .. "_GATE_TILING2_PART"
                },
		BasementFillerList = {
			CoveredBridgePrefix .. "_GATE_TILING3_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
	AssetBuildingFunction = CoveredBridgePrefix .. "_GATE_FUNCTION"
})
coveredBridge:register({
		DataType = "BUILDING_FUNCTION_WORKPLACE",
                Id = CoveredBridgePrefix .. "_GATE_FUNCTION",
		WorkerCapacity = 2,
                RelatedJob = { Job = "SOLDIER", Behavior = _CoveredBridgeTagBehaviorWork }
})

coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_GATE_TOP_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
                   DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                   CoreObjectPrefab = CoveredBridgePrefix .. "_GATE_TOP_PREFAB"
        },
        Cost = {
	    UpkeepCost = {
		{ Resource = "GOLD", Quantity = 1 }
	    },
	    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
		{ Resource = "STONE", Quantity = 2 }
	    }
        },
        EstateSplendor = {
	    { Estate = "KINGDOM", Quantity = 0.1 }
        }
})

coveredBridge:register({
		DataType = "BUILDING_FUNCTION_ASSIGNABLE",
                Id = CoveredBridgePrefix .. "_ASSIGNABLE_FUNCTION",
                AuthorizedFunctionList = CoveredBridgePrefix .. "_FUNCTION_LIST"
})

coveredBridge:register({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = CoveredBridgePrefix .. "_FUNCTION_LIST",
                    AssetFunctionList = {
                        CoveredBridgePrefix .. "_TAILOR_FUNCTION",
                        "BUILDING_FUNCTION_FOOD_MARKET",
                        "BUILDING_FUNCTION_GOODS_MARKET",
                        "BUILDING_FUNCTION_LUXURY_MARKET"
                }
})

coveredBridge:register({
		DataType = "BUILDING_FUNCTION_MARKET",
                Id = CoveredBridgePrefix .. "_TAILOR_FUNCTION",
                Name = CoveredBridgePrefix .. "_TAILOR_FUNCTION_NAME",
                TypeList = { "GOODS" },
                IsOverrideCapacity = true,
                CapacityPerSlot = 20,
		WorkerCapacity = 1,
                RelatedJob = { Job = "TAILOR", Behavior = _CoveredBridgeTagBehaviorWork },
                InputInventoryCapacity = {
			{ Resource = "WOOLEN_CLOTH", Quantity = 20 }
                },
                ResourceListNeeded = {
			{ Resource = "WOOLEN_CLOTH", Quantity = 1 }
                },
                ResourceProduced = {
			{ Resource = "CLOTHES", Quantity = 1 }
                }
})

coveredBridge:dofile("scripts/compCoveredBridge.lua")

-- Register the house decoration part
function registerHousePart(_nameFbx, _namePart, _nameAsset)
    coveredBridge:registerAssetId("models/" .. _nameFbx .. ".fbx/Prefab/coveredBridge/" .. _namePart .. "Part", CoveredBridgePrefix .. "_" .. _nameAsset .. "_PREFAB")
    coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_" .. _nameAsset .. "_PART",
	Name = CoveredBridgePrefix .. "_" .. _nameAsset .. "_NAME",
	Description = CoveredBridgePrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_" .. _nameAsset .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	},
  	AssetBuildingFunction = CoveredBridgePrefix .. "_ASSIGNABLE_FUNCTION" 
    })

    coveredBridge:registerPrefabComponent("models/" .. _nameFbx .. ".fbx/Prefab/coveredBridge/" .. _namePart .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	HasBuildingZone = true, -- to build it standalone
	BuildingZone = { 0, 0 }
    })

    coveredBridge:configurePrefabFlagList("models/" .. _nameFbx .. ".fbx/Prefab/coveredBridge/" .. _namePart .. "Part", { "BRIDGE" })

    coveredBridge:registerPrefabComponent("models/" .. _nameFbx .. ".fbx/Prefab/coveredBridge/" .. _namePart .. "Part/PATH_A_0", {
	DataType = CoveredBridgePrefix .. "_PATH_A0_COMP",
	Enabled = true
    })
    coveredBridge:registerPrefabComponent("models/" .. _nameFbx .. ".fbx/Prefab/coveredBridge/" .. _namePart .. "Part/PATH_B_0", {
	DataType = CoveredBridgePrefix .. "_PATH_B0_COMP",
	Enabled = true
    })
end

registerHousePart("coveredBridgeDecoration", "House", "HOUSE")
registerHousePart("coveredBridgeDecoration", "Food", "RED")
registerHousePart("coveredBridgeDecoration", "Goods", "GREEN_A")
registerHousePart("coveredBridgeDecoration", "Luxury", "BLUE")
registerHousePart("coveredBridgeDecoration", "Tailor", "GREEN_B")
registerHousePart("coveredBridgeAddition", "Hat", "RED_WINE")
registerHousePart("coveredBridgeAddition", "Knife", "CARBON")

-- Register the center coverage decoration part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_CENTER_COVERAGE_PART",
	Name = CoveredBridgePrefix .. "_CENTER_COVERAGE_NAME",
	Description = CoveredBridgePrefix .. "_CENTER_COVERAGE_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_CENTER_COVERAGE_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
})

-- Register the start or end coverage decoration part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = CoveredBridgePrefix .. "_START_END_COVERAGE_PART",
	Name = CoveredBridgePrefix .. "_START_END_COVERAGE_NAME",
	Description = CoveredBridgePrefix .. "_START_END_COVERAGE_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_START_END_COVERAGE_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 2 }
		}
	}
})

--------------------------------------------------------------------------

-- Override building part components properties (generated by the building asset processor) to add building zones

coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/CenterPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/StartPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/EndPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/StartSlopePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/EndSlopePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeGate.fbx/Prefab/coveredBridge/Gate/step_0_RootPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	BuildingZone = { 8, 4 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/CenterCoveragePart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/StartEndCoveragePart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
--------------------------------------------------------------------------

-- Configure all BRIDGE colliders

coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/CenterPart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/StartPart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/EndPart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/StartSlopePart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/EndSlopePart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridgeGate.fbx/Prefab/coveredBridge/Gate/step_3_TopPart", { "PLATFORM" })
